/*=== Copyright (c) 2000 Sollae Systems Co.,Ltd. ===*/
/*=============== All rights reserved ==============*/

/*
modification history:
05.Jun.2000 yyc  initial touch
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <setjmp.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

static jmp_buf jmpto;
static int sig_msg;
static unsigned char recv_buf[1024];

static void sig_call(int sig)
{
	sig_msg = sig;
	longjmp(jmpto, 1);
}

void main(void)
{
	struct sockaddr_in sock_in;
	int sockfd, newfd, addr_len, len, recv_count;
	FILE *fp;

	/* redirect SIGINT(ctrl-C pressed) handler to sig_call() function */
	signal(SIGINT, sig_call);

	/* redirect SIGTERM(user kill request) handler to sig_call() function. */
	signal(SIGTERM, sig_call);

	if(setjmp(jmpto))
	{
		/* if signal is generated, context enters here from sig_call(). */
		if(sig_msg == SIGINT)
			printf("signal: SIGINT\n");
		if(sig_msg == SIGTERM)
			printf("signal: SIGTERM\n");
		goto _exit_cod;
	}

	/* formal listening sequence on UNIX: socket() -> bind() -> listen() */
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if(sockfd < 0)
	{
		perror("socket:");
		return;
	}
	memset((void *)&sock_in, 0, sizeof(sock_in));
	sock_in.sin_family      = AF_INET;
	sock_in.sin_addr.s_addr = htonl(INADDR_ANY);
	sock_in.sin_port        = htons(1470);
	if(bind(sockfd, (struct sockaddr *)&sock_in, sizeof(sock_in)) < 0)
	{
		perror("bind:");
		close(sockfd);
		return;
	}
	if(listen(sockfd, 1) < 0)
	{
		perror("listen:");
		close(sockfd);
		return;
	}

	/* create file to record stream generated from ezTCP */
	fp = fopen("cod.log", "w+t");
	if(fp == NULL)
	{
		perror("fopen:");
		close(sockfd);
	}

_again:
	newfd = -1;
	newfd = accept(sockfd, (struct sockaddr *)&sock_in, &addr_len);
	if(newfd < 0)
	{
		perror("accept:");
		goto _exit_cod;
	}

	/* print notification message */
	printf("connect from %s\n", inet_ntoa(sock_in.sin_addr));

	recv_count = 0;
	for(;;)
	{
		len = recv(newfd, recv_buf, 1024, 0);
		if(len < 0)
		{
			perror("recv:");
			break;
		}
		if(!len)
		{ /* if connection is closed by peer, recv() function returns zero. */
			printf("connection closed.\n");
			break;
		}
		recv_count += len;
		fwrite(recv_buf, len, 1, fp);
	}
	printf("%dbytes received.\n", recv_count);

	/* flush file buffer cache */
	fflush(fp);

	/* close current connection */
	close(newfd);

	/* wait another request again */
	goto _again;

_exit_cod:
	fclose(fp);
	close(sockfd);
	if(newfd > 0)
		close(newfd);
}
